unit Main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, Buttons, Gauges, Menus,
  VBScript_RegExp_55_TLB;

//-----------------------------------------------------------------------------
type
    TPageInfo = record
        URL: string;
        Host: string;
        Page: string;
        Status: string;
        HttpCode: integer;
        ContentType: string;
        Charset: string;
        Title: string;
        Description: string;
    end;

type
  TMainForm = class(TForm)
    MainMenu1: TMainMenu;
    MenuFile: TMenuItem;
    MenuExit: TMenuItem;
    MenuMisc: TMenuItem;
    MenuInfo: TMenuItem;
    MenuStayOnTop: TMenuItem;
    Panel1: TPanel;
    BitBtn1: TBitBtn;
    Memo1: TMemo;
    Splitter1: TSplitter;
    Memo2: TMemo;
    procedure FormCreate(Sender: TObject);
    procedure MenuInfoClick(Sender: TObject);
    procedure MenuExitClick(Sender: TObject);
    procedure MenuStayOnTopClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure BitBtn1Click(Sender: TObject);
  private
    FRegBuf: TRegExp;
    FStrBuf: TStringList;
    function GetPageInformation(const URL: string): TPageInfo;
  public

  end;

var
  MainForm: TMainForm;

implementation

{$R *.dfm}

uses
    StrUtils, Jconvert, Info, Http;

//-----------------------------------------------------------------------------
//  2005.09.03  RXgN^
procedure TMainForm.FormCreate(Sender: TObject);
begin
    SetFormCaption(Self);
    FRegBuf := TRegExp.Create(nil);
    FStrBuf := TStringList.Create();
end;

//-----------------------------------------------------------------------------
//  2005.09.03  fXgN^
procedure TMainForm.FormDestroy(Sender: TObject);
begin
    FRegBuf.Free;
    FStrBuf.Free;
end;

//-----------------------------------------------------------------------------
//  2005.09.06  Tu}b`1ڂ߂
function GetSubMatchItem0(RegBuf: TRegExp; const Data, Pattern: string): string;
var
    Matches, SubMatches: OleVariant;
begin
    Result := '';
    RegBuf.Pattern := Pattern;
    RegBuf.Global := false;            //  ŏ
    RegBuf.IgnoreCase := true;         //  啶E
    Matches := RegBuf.Execute(Data);   //  
    if (Matches.Count > 0) then
    begin
        SubMatches := Matches.Item[0].SubMatches;
        if (SubMatches.Count > 0) then
        begin
            Result := SubMatches.Item[0];
        end;
    end;
end;

//-----------------------------------------------------------------------------
//  2005.09.06  R[hϊ
procedure ConvertCoding(var Data: string; const Charset: string);
begin
    //  Shift_JIS
    if (AnsiContainsText(Charset, 'Shift_JIS')) then
    begin
        //  ̂܂
    end

    //  xsjis
    else if (AnsiContainsText(Charset, 'xsjis')) then
    begin
        //  ̂܂
    end

    //  UTF-8
    else if (AnsiContainsText(Charset, 'UTF-8')) then
    begin
        Data := Utf8ToAnsi(Data);
    end

    //  EUC̏ꍇ
    else if (AnsiContainsText(Charset, 'euc')) then
    begin
        Data := euc2sjis(Data);
    end

    //  s̏ꍇ
    else
    begin
        Data := ConvertJCode(Data, SJIS_OUT);
    end;
end;

//-----------------------------------------------------------------------------
//  2005.09.06  Pagȅ擾
function TMainForm.GetPageInformation(const URL: string): TPageInfo;

var
    ss: TStringStream;
    host, page, Command: string;
    Data, Pattern: string;
begin
    //  
    FillChar(Result, SizeOf(Result), #0);

    Result.URL := URL;

    //  URL
    if (not SplitUrl(Result.URL, host, page)) then Exit;

    ss := nil;
    try
        try
            while (true) do
            begin
                Result.Host := host;
                Result.Page := page;

                //  R}h쐬
                Command := Format(
                    'GET %s HTTP/1.0'#13#10
                  + 'HOST: %s:80'#13#10
                  + 'User-Agent: Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 5.0)'
                  + #13#10#13#10, [page, host]);

                //  Xg[쐬
                ss := TStringStream.Create('');

                //  _E[h
                if (not GetHttpPageFromHost(host, Command, ss)) then break;

                //  wb_̃Xg[f[^𕶎
                Data := ss.DataString;

                //  pς
                FreeAndNil(ss);

                //  Xe[^Xo
                Pattern := '(^HTTP/[0-9]\.[0-9][\s]+[0-9]+[\s]+[^\s]*)';
                Result.Status := GetSubMatchItem0(FRegBuf, Data, Pattern);

                //  Xe[^XR[ho
                Pattern := '^HTTP/[0-9]\.[0-9][\s]+([0-9]+)[\s]+[^\s]*';
                Result.HttpCode := StrToIntDef(GetSubMatchItem0(FRegBuf, Result.Status, Pattern), -1);

                //  Ȃ񂶂Ⴑ႟
                if (Result.HttpCode < 0) then break;

                //  ړH
                if ((Result.HttpCode = 301) or (Result.HttpCode = 302)) then
                begin
                    //  Location o
                    Pattern := 'Location:[\s]*([^\s]*)';
                    page := GetSubMatchItem0(FRegBuf, Data, Pattern);

                    //  http n܂ĂH
                    if (AnsiStartsText('http://', page)) then
                    begin
                        //  URLXV
                        Result.URL := page;

                        //  ߂URL
                        if (not SplitUrl(Result.URL, host, page)) then Exit;
                    end;

                    //  蒼
                    continue;
                end;

                //  200ȊO͂ǂĂ̂
                if (Result.HttpCode <> 200) then break;

                //  Content-Typeo
                Pattern := 'Content-Type:[\s]*([^;\s]*)';
                Result.ContentType := GetSubMatchItem0(FRegBuf, Data, Pattern);

                //  Content-Type META^O
                if (Result.ContentType = '') then
                begin
                    Pattern := 'http-equiv=["]?content-type"[\s]+content=["]?([^;"\s]*)';
                    Result.ContentType := Trim(GetSubMatchItem0(FRegBuf, Data, Pattern));
                end;

                //  charseto(T[o[X|XMETA^Oɔ)
                Pattern := ';[\s]*charset[\s]*=[\s]*([^\s">]*)';
                Result.Charset := Trim(GetSubMatchItem0(FRegBuf, Data, Pattern));

                //  R[hϊ
                ConvertCoding(Data, Result.Charset);

//DEBUGDEBUGDEBUGDEBUGDEBUGDEBUGDEBUGDEBUGDEBUGDEBUGDEBUGDEBUGDEBUGDEBUGDEBUG
FStrBuf.Text := Data;
FStrBuf.SaveToFile(ChangeFileExt(Application.ExeName, host + '.txt'));
//DEBUGDEBUGDEBUGDEBUGDEBUGDEBUGDEBUGDEBUGDEBUGDEBUGDEBUGDEBUGDEBUGDEBUGDEBUG

                //  ^Cg^O̔
                Pattern := '<title[^>]*>([^<]*)</title>';
                Result.Title := GetSubMatchItem0(FRegBuf, Data, Pattern);

                //  ^CgȂȂsĂ邩Ƃ݂ȂėƂ
                if (Result.Title = '') then
                begin
                    Data := StringReplace(Data, #13, '', [rfReplaceAll]);
                    Data := StringReplace(Data, #10, '', [rfReplaceAll]);
                end;

                //  ̔-Xebv1
                Pattern := '<meta[\s]*name[\s]*=[\s]*"?description"?[\s]*([^>]*)>';
                Result.Description := Trim(GetSubMatchItem0(FRegBuf, Data, Pattern));

                //  ̔-Xebv2
                Pattern := 'content[\s]*="?([^">]*)"?';
                Result.Description := GetSubMatchItem0(FRegBuf, Result.Description, Pattern);

                //  ܂
                break;
            end;
        finally
            FreeAndNil(ss);
        end;
    except
        //
    end;
end;

//  ---------------------------------------------------------------------------
//  s
procedure TMainForm.MenuInfoClick(Sender: TObject);
begin
    OpenInfoForm();
end;

procedure TMainForm.MenuExitClick(Sender: TObject);
begin
    Self.Close();
end;

procedure TMainForm.MenuStayOnTopClick(Sender: TObject);
begin
    if (MenuStayOnTop.Checked) then
        Self.FormStyle := fsStayOnTop
    else
        Self.FormStyle := fsNormal;
end;

procedure TMainForm.BitBtn1Click(Sender: TObject);
var
    i: integer;
    PageInfo: TPageInfo;
begin
    for i := 0 to Memo1.Lines.Count - 1 do
    begin
        PageInfo := GetPageInformation(Memo1.Lines.Strings[i]);
        Memo2.Lines.Add(StringOfChar('-', 50));
        Memo2.Lines.Add('URL         :' + PageInfo.URL);
        Memo2.Lines.Add('Host        :' + PageInfo.Host);
        Memo2.Lines.Add('Page        :' + PageInfo.Page);
        Memo2.Lines.Add('Status      :' + PageInfo.Status);
        Memo2.Lines.Add('ContentType :' + PageInfo.ContentType);
        Memo2.Lines.Add('Charset     :' + PageInfo.Charset);
        Memo2.Lines.Add('Title       :' + PageInfo.Title);
        Memo2.Lines.Add('Description :' + PageInfo.Description);
    end;
end;

end.

